Marketo Anypoint&trade; Studio Connector Demo
==========================================

## Introduction

The current demo application provides a workflow to invoke Marketo's API operations for Leads and Programs.

## Prerequisites

* Java 7 
* Anypoint Studio 5.4.x
* Mule Runtime 3.7.0 EE
* DataWeave
* Access to a Marketo instance

## Import the project

* Go to **File > Import**
* Select **Anypoint Studio Project from External Location** (under the parent folder "Anypoint Studio")

![Import Menu](images/mk_import_menu.png)

* Provide the root path to the demo project folder.
* Select Mule Runtime (3.7.x EE).

![File Selection](images/mk_import_file.png)

* Click **Finish**.  
* Set Marketo credentials inside the file `src/main/app/mule-app.properties`.

```
marketo.clientId=1<CLIENT_ID>
marketo.clientSecret=<CLIENT_SECRET>
marketo.restEndpointUrl=<ENDPOINT_REST_URL> e.g https://<APP_NAME>.mktorest.com/rest
```

* Inside the Global Elements view, double-click the Marketo Configuration component and click "Test Connection".
![Connection Config](images/mk_connection_config.png)

* A successful message should pop-up.
![Test Connection](images/mk_connection_test.png)

If you receive the error **`Unsupported major/minor version 51.0`**, it indicates that you are running with a 1.6 JRE. Ensure that you are running with Java 1.7.

> For further information, refer to the [Marketo Connector Documentation Guide](http://docs.mulesoft.com/mule-user-guide/v/3.7/marketo-connector).

* Select **Run as Mule Application**. If when deploying the application, Studio throws the exception **`Caused by: org.xml.sax.SAXParseException: cvc-complex-type.2.4.a: Invalid content was found starting with element 'marketo:config' (...)`**, 
the project might be missing the connector library. To verify this, go to the the **Package Explorer** view, and search for **Marketo[v2.0.0]**.  If it doesn't exist, follow these guidelines:

	1. Right-click on the project **demo-leads-and-programs**.
	2. Go to **Build Path > Add Libraries**.
	3. Inside the **Anypoint Connector Dependencies**, check the **Marketo** option. If there is more than one listed, it is advisable you use the newest version of the connector.
		![Import Connector Library](images/mk_import_connector.png)
	4. The connector dependency should appear in the project's build path:
		![Connector Package Explorer](images/mk_import_package_explorer.png)
	
* Open a browser and access the URL `http://localhost:8081`. You should see the demo application deployed:

![Demo Index](images/mk_demo_home.png)

## About the flows

**1. Html_Form_Flow:** renders the HTML form with a `parseTemplate` component.

![Flow HMTL Form](images/mk_flow_htmlForm.png)

**2. Import_Leads_Flow:** inserts or updates one or more Leads from a text file. Supported formats are: CSV, TSV and SSV.

![Flow Import Leads](images/mk_flow_importLeads.png)

###### Code Snippet
```
<marketo:import-lead config-ref="Marketo__Configuration" file-ref="#[payload]" listId="#[flowVars.listId]" doc:name="Import Leads"/>
```

**3. CreateOrUpdate_Lead_Flow:** inserts or updates a single Lead through a web form and assigns it to a given Static list. To maintain the simplicity of the demo, a single List ID is provided, but the processor can take multiple IDs. 

![Flow Create or Update Lead](images/mk_flow_createOrUpdateLead.png)

###### Code Snippet
```
<marketo:create-or-update-lead config-ref="Marketo__Configuration" doc:name="Create Lead"/>
```

**4. Get_Multiple_Lists_Flow:** auxiliary flow that retrieves all Static lists **only** (Smart lists are currently not supported by Marketo's REST API).

![Flow Get Multiple Lists](images/mk_flow_getMultipleLists.png)

###### Code Snippet
```
<marketo:get-multiple-lists config-ref="Marketo__Configuration" doc:name="Get Multiple Lists"/>
```

**6. Get_Multiple_Leads_By_List_Flow:** retrieves the Leads that belong to a given Static list.

![Flow Get Multiple Leads bt List](images/mk_flow_getMultipleLeadsByList.png)

###### Code Snippet
```
<marketo:get-multiple-leads-by-list-id config-ref="Marketo__Configuration" listId="#[payload]" doc:name="Get Multiple Leads by List Id"/>
```

**7. Browse_Folders_Flow:** auxiliary flow that retrieves the first level of folders inside the root folder "Marketing Activities".

![Flow Browse Folders](images/mk_flow_browseFolders.png)

###### Code Snippet
```
<marketo:browse-folders config-ref="Marketo__Configuration" root-ref="#[payload.folderId]" doc:name="Browse Folders"/>
```

**8. Delete_Lead_Flow:** deletes an existing Lead given its ID.

![Flow Delete Lead](images/mk_flow_deleteLead.png)

###### Code Snippet
```
<marketo:delete-lead config-ref="Marketo__Configuration" doc:name="Delete Lead"/>
```

**9. Create_Program_Flow:** saves a new Email program inside a given folder.

![Flow Create Program](images/mk_flow_createProgram.png)

###### Code Snippet
```
<marketo:create-program config-ref="Marketo__Configuration" folder-ref="#[payload.folder]" channel="#[payload.channel]" programName="#[payload.programName]" doc:name="Create Program" type="EMAIL"/>
```

**10. Delete_Program_Flow:** deletes an existing Program given its ID.

![Flow Delete Lead](images/mk_flow_deleteProgram.png)

###### Code Snippet
```
<marketo:delete-program-by-id config-ref="Marketo__Configuration" doc:name="Delete Lead"/>
```

## Run the demo

### Leads Endpoint

* Click the **"Leads"** menu.

#### Import Lead

![Demo Import Lead](images/mk_form_leads_import.png)

* Click on the tab **"Import"**.
* Select the file that contains the information of the Leads to be imported. Accepted types include CSV, TSV or SSV. A sample CSV file has been attached to the demo inside the directory `${DEMO}/data/ImportLeads.csv`. It contains the information of 3 Leads:

```
lastName,firstName,title,company,email,phone,leadStatus,leadScore,leadSource,updatedAt
The Mule,Max,CEO,Muley Inc,max.themule@mulesoft.com,,,,,2016-04-07 11:22:33
Charleston,Charles,Vice President Junior,Muley Inc, charles.charleston@mulesoft.com,,,,,2016-04-07 11:22:33
Jenkins, Mister,Product Manager,Muley Inc,mister.jenkins2@mulesoft.com,,,,,2016-04-07 11:22:33
```

* Optionally, specify the ID of a Static list.
* Click the button **"Upload to Marketo"** and wait a few moments to finish processing.
* A pop-up message will indicate if the operation was successful or not. 

#### Create or Update

![Demo Create or Update Lead](images/mk_form_leads_createOrUpdate.png)

* Click on the tab **"Create"**.
* Fill in the following required fields: 
	* Company
	* Email
	* Last Name
	* First Name
* Select an operation from the "Operation" menu: Create Only, Update Only, Create or Update, Create Duplicate.
* Select a list from the "Add to List" menu.
* Click the button **"Create or Update"** and wait a few moments to finish processing.
* A pop-up message will indicate if the operation was successful or not. 

* Optionally, click the button **"Display Members of List"** to show the complete list of Leads in a separate pop-up window.

![Demo Get Leads](images/mk_form_leads_getMembersOfList.png)

#### Delete

![Demo Delete Lead](images/mk_form_leads_delete.png)

* Click on the tab **"Delete"**.
* Enter the ID number of the Lead.
* Click the button **"Delete Lead"** and wait a few moments to finish processing.
* A pop-up message will indicate if the operation was successful or not. 

---

### Program Endpoint

* Click the **"Programs"** menu.

#### Create

![Demo Create Program](images/mk_form_program_create.png)

* Click on the tab **"Create"**.
* Select a folder in the **Campaign Folder** dropdown list. This will be the parent folder of the program.
* Provide a name for the new program in the **Program Name** field.
* Select a type in the **Program Type** dropdown list.
* Select a channel in the **Channel** dropdown list.
* Click the button **"Create"** and wait a few moments to finish processing.
* A pop-up message will indicate if the operation was successful or not. 

#### Delete

![Demo Delete Program](images/mk_form_program_delete.png)

* Click on the tab **"Delete"**.
* Enter the ID number of the Program.
* Click the button **"Delete Program"** and wait a few moments to finish processing.
* A pop-up message will indicate if the operation was successful or not. 


